package com.rthoni.stssaguenay.ui.activities;

import android.app.ProgressDialog;
import android.content.Context;
import android.content.DialogInterface;
import android.content.Intent;
import android.os.Bundle;
import android.support.design.widget.NavigationView;
import android.support.multidex.MultiDex;
import android.support.v4.app.FragmentManager;
import android.support.v4.widget.DrawerLayout;
import android.support.v7.app.ActionBar;
import android.support.v7.app.ActionBarDrawerToggle;
import android.support.v7.app.AlertDialog;
import android.support.v7.app.AppCompatActivity;
import android.support.v7.widget.Toolbar;
import android.text.InputType;
import android.view.Menu;
import android.view.MenuItem;
import android.view.View;
import android.widget.EditText;
import android.widget.TextView;
import android.widget.Toast;

import com.facebook.CallbackManager;
import com.facebook.FacebookCallback;
import com.facebook.FacebookException;
import com.facebook.FacebookSdk;
import com.facebook.GraphRequest;
import com.facebook.GraphResponse;
import com.facebook.login.LoginManager;
import com.facebook.login.LoginResult;
import com.facebook.login.widget.LoginButton;
import com.github.clans.fab.FloatingActionButton;
import com.github.clans.fab.FloatingActionMenu;
import com.google.android.gms.ads.AdRequest;
import com.google.android.gms.ads.AdView;
import com.google.android.gms.ads.MobileAds;
import com.luticate.utils.business.LuPromise;
import com.luticate.utils.business.LuRequest;
import com.luticate.utils.dbo.LuVoidDbo;
import com.rthoni.stssaguenay.R;
import com.rthoni.stssaguenay.business.STSBusiness;
import com.rthoni.stssaguenay.business.UsersSettingsBusiness;
import com.rthoni.stssaguenay.dbo.StopsDbo;
import com.rthoni.stssaguenay.dbo.UserFavouriteStopsDbo;
import com.rthoni.stssaguenay.dbo.UserDbo;
import com.rthoni.stssaguenay.dbo.UsersSettingsDbo;
import com.rthoni.stssaguenay.ui.fragments.HomeFragment;
import com.rthoni.stssaguenay.ui.fragments.StopViewerFragment;

import org.json.JSONException;
import org.json.JSONObject;

import java.util.List;

import butterknife.BindView;
import butterknife.ButterKnife;

public class MainActivity extends AppCompatActivity implements NavigationView.OnNavigationItemSelectedListener {

    public static int ADD_FAVOURITE_STOP_REQUEST_CODE = 1;

    private boolean _goToHome = false;

    @BindView(R.id.fab)
    FloatingActionMenu _fab;

    @BindView(R.id.fab_item_add_map)
    FloatingActionButton _fabMap;

    @BindView(R.id.fab_item_add_list)
    FloatingActionButton _fabList;

    @BindView(R.id.nav_view)
    NavigationView _navigationView;

    private LoginButton _btnLogin;

    private TextView _textUserName;

    private TextView _textUserEmail;

    private TextView _textHelpMessage;

    private CallbackManager _callbackManager;

    private ActionBar _actionBar;

    private FragmentManager _fragmentManager;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);

        LuRequest.init(this);

        FacebookSdk.sdkInitialize(getApplicationContext());

        setContentView(R.layout.activity_main);
        ButterKnife.bind(this);

        MobileAds.initialize(getApplicationContext(), getString(R.string.ad_init));
        AdView mAdView = (AdView) findViewById(R.id.adView);
        AdRequest adRequest = new AdRequest.Builder().build();
        mAdView.loadAd(adRequest);

        Toolbar toolbar = (Toolbar) findViewById(R.id.toolbar);
        setSupportActionBar(toolbar);

        DrawerLayout drawer = (DrawerLayout) findViewById(R.id.drawer_layout);
        ActionBarDrawerToggle toggle = new ActionBarDrawerToggle(
                this, drawer, toolbar, R.string.navigation_drawer_open, R.string.navigation_drawer_close);
        drawer.setDrawerListener(toggle);
        toggle.syncState();

        _navigationView.setNavigationItemSelectedListener(this);

        View headerView = _navigationView.getHeaderView(0);
        _textUserName = ButterKnife.findById(headerView, R.id.textUserName);
        _textUserEmail = ButterKnife.findById(headerView, R.id.textUserEmail);
        _textHelpMessage = ButterKnife.findById(headerView, R.id.text_help_message);

        _fabMap.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                goToAddStop(true);
            }
        });

        _fabList.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                goToAddStop(false);
            }
        });

        _callbackManager = CallbackManager.Factory.create();
        _btnLogin = ButterKnife.findById(headerView, R.id.btnLogin);
        _btnLogin.setReadPermissions("email");
        _btnLogin.registerCallback(_callbackManager, new FacebookCallback<LoginResult>() {
            @Override
            public void onSuccess(LoginResult loginResult) {
                GraphRequest request = GraphRequest.newMeRequest(
                        loginResult.getAccessToken(),
                        new GraphRequest.GraphJSONObjectCallback() {
                            @Override
                            public void onCompleted(JSONObject object, GraphResponse response) {
                                try {
                                    UserDbo userDbo = new UserDbo();
                                    userDbo.fromJson(object);
                                    onUserLogged(userDbo);
                                } catch (JSONException e) {
                                    e.printStackTrace();
                                }
                            }
                        });
                Bundle parameters = new Bundle();
                parameters.putString("fields", "id,email,name");
                request.setParameters(parameters);
                request.executeAsync();
                loginResult.getAccessToken();
            }

            @Override
            public void onCancel() {

            }

            @Override
            public void onError(FacebookException error) {
                Toast.makeText(MainActivity.this, error.toString(), Toast.LENGTH_LONG).show();
            }
        });


        _fragmentManager = getSupportFragmentManager();
        _fragmentManager.addOnBackStackChangedListener(new FragmentManager.OnBackStackChangedListener() {
            @Override
            public void onBackStackChanged() {
                if (_fragmentManager.getBackStackEntryCount() == 0) {
                    goToHome();
                }
            }
        });

        _actionBar = getSupportActionBar();

        updateLoginState();
        goToHome();
    }

    @Override
    protected void attachBaseContext(Context newBase) {
        super.attachBaseContext(newBase);
        MultiDex.install(this);
    }

    @Override
    public boolean onSupportNavigateUp() {
        if (_fragmentManager.getBackStackEntryCount() == 0) {
            return super.onSupportNavigateUp();
        }
        _fragmentManager.popBackStack();
        return true;
    }

    @Override
    protected void onResume() {
        if (_goToHome) {
            _goToHome = false;
            goToHome();
        }
        super.onResume();
    }

    public void goToHome()
    {
        _fab.setVisibility(View.VISIBLE);
        HomeFragment f = new HomeFragment();
        f.setOnFavouriteStopClicked(new LuPromise.LuConsumer<UserFavouriteStopsDbo>() {
            @Override
            public void execute(UserFavouriteStopsDbo data) {
                goToStop(data.getStop());
            }
        });
        getSupportFragmentManager().beginTransaction().replace(R.id.container, f).commit();
    }

    public void goToStop(StopsDbo stopsDbo)
    {
        _fab.setVisibility(View.GONE);
        StopViewerFragment f = new StopViewerFragment();
        f.setStopsDbo(stopsDbo);
        getSupportFragmentManager()
                .beginTransaction()
                .replace(R.id.container, f)
                .addToBackStack("StopViewerFragment")
                .commit();
    }

    public void goToAddStop(boolean map)
    {
        Intent intent = new Intent(this, StopPickerActivity.class);
        intent.putExtra(StopPickerActivity.MAP_EXTRA_NAME, map);
        startActivityForResult(intent, ADD_FAVOURITE_STOP_REQUEST_CODE);
    }

    @Override
    protected void onActivityResult(int requestCode, int resultCode, Intent data) {
        if (requestCode == ADD_FAVOURITE_STOP_REQUEST_CODE && resultCode == RESULT_OK) {
            UserFavouriteStopsDbo favouriteStopDbo = new UserFavouriteStopsDbo();
            try {
                JSONObject obj = new JSONObject(data.getStringExtra(StopPickerActivity.STOP_EXTRA_NAME));
                favouriteStopDbo.fromJson(obj);
            } catch (JSONException e) {
                e.printStackTrace();
                return;
            }
            List<UserFavouriteStopsDbo> favouriteStopDbos = STSBusiness.getFavouriteStops(this);
            favouriteStopDbos.add(favouriteStopDbo);
            STSBusiness.setFavouriteStops(this, favouriteStopDbos);
            _goToHome = true;
        }
        else {
            _callbackManager.onActivityResult(requestCode, resultCode, data);
        }
        super.onActivityResult(requestCode, resultCode, data);
    }

    @Override
    public boolean onNavigationItemSelected(MenuItem item) {
        if (item.getItemId() == R.id.nav_restore) {
            restoreSettings();
        }
        else if (item.getItemId() == R.id.nav_backup) {
            backupSettings();
        }
        else if (item.getItemId() == R.id.nav_logout) {
            LoginManager.getInstance().logOut();
            STSBusiness.setLoggedUser(this, null);
            updateLoginState();
        }
        else {
            return false;
        }
        return true;
    }

    public void updateLoginState()
    {
        UserDbo user = STSBusiness.getLoggedUser(this);
        _btnLogin.setVisibility(user == null ? View.VISIBLE : View.GONE);
        Menu menu = _navigationView.getMenu();
        for (int i = 0; i < menu.size(); i++) {
            menu.getItem(i).setVisible(user != null);
        }
        _textUserName.setText(user == null ? "" : user.getName());
        _textUserName.setVisibility(user == null ? View.GONE : View.VISIBLE);
        _textUserEmail.setText(user == null ? "" : user.getEmail());
        _textUserEmail.setVisibility(user == null ? View.GONE : View.VISIBLE);
        _textHelpMessage.setVisibility(user == null ? View.VISIBLE : View.GONE);
    }

    public void onUserLogged(UserDbo userDbo)
    {
        STSBusiness.setLoggedUser(this, userDbo);
        updateLoginState();
    }

    public LuPromise<String> askPassword(boolean encrypt)
    {
        final LuPromise<String> promise = new LuPromise<>();

        AlertDialog.Builder builder = new AlertDialog.Builder(this);
        builder.setTitle(R.string.encryption_password);
        builder.setMessage(encrypt ? R.string.encryption_password_encrypt : R.string.encryption_password_decrypt);

        final EditText input = new EditText(this);
        input.setInputType(InputType.TYPE_CLASS_TEXT | InputType.TYPE_TEXT_VARIATION_PASSWORD);
        builder.setView(input);

        builder.setPositiveButton(android.R.string.ok, new DialogInterface.OnClickListener() {
            @Override
            public void onClick(DialogInterface dialog, int which) {
                promise.resolve(input.getText().toString());
            }
        });
        builder.setNegativeButton(android.R.string.cancel, new DialogInterface.OnClickListener() {
            @Override
            public void onClick(DialogInterface dialog, int which) {
                dialog.cancel();
            }
        });

        builder.show();
        return promise;
    }

    public void restoreSettings()
    {
        final ProgressDialog progressDialog = new ProgressDialog(MainActivity.this);
        progressDialog.setIndeterminate(true);
        progressDialog.setCancelable(false);
        progressDialog.show();
        progressDialog.setTitle(R.string.loading);
        progressDialog.setMessage(getString(R.string.settings_restoring));

        UserDbo user = STSBusiness.getLoggedUser(MainActivity.this);
        UsersSettingsBusiness.get(STSBusiness.getConfig(this), user.getId())
                .then(new LuPromise.LuConsumer<String>() {
                    @Override
                    public void execute(final String data) {
                        progressDialog.dismiss();
                        if (data == null) {
                            Toast.makeText(MainActivity.this, R.string.encryption_empty, Toast.LENGTH_LONG).show();
                            return;
                        }
                        askPassword(false).then(new LuPromise.LuConsumer<String>() {
                            @Override
                            public void execute(String password) {
                                UsersSettingsDbo settingsDbo = UsersSettingsBusiness.decryptSettings(data, password);
                                if (settingsDbo == null) {
                                    Toast.makeText(MainActivity.this, R.string.encryption_bad_password, Toast.LENGTH_LONG).show();
                                }
                                else {
                                    Toast.makeText(MainActivity.this, R.string.settings_restored, Toast.LENGTH_LONG).show();
                                    STSBusiness.setUserSettings(MainActivity.this, settingsDbo);
                                    goToHome();
                                }
                            }
                        });
                    }
                }, new LuPromise.LuConsumer<LuPromise.LuPromiseError>() {
                    @Override
                    public void execute(LuPromise.LuPromiseError data) {
                        progressDialog.dismiss();
                        Toast.makeText(MainActivity.this, data.toString(), Toast.LENGTH_LONG).show();
                    }
                });
    }

    public void backupSettings()
    {
        askPassword(true).then(new LuPromise.LuConsumer<String>() {
            @Override
            public void execute(String password) {
                UserDbo user = STSBusiness.getLoggedUser(MainActivity.this);
                UsersSettingsDbo settingsDbo = STSBusiness.getUserSettings(MainActivity.this);
                String data = UsersSettingsBusiness.encryptSettings(settingsDbo, password);
                final ProgressDialog progressDialog = new ProgressDialog(MainActivity.this);
                progressDialog.setIndeterminate(true);
                progressDialog.setCancelable(false);
                progressDialog.show();
                progressDialog.setTitle(R.string.loading);
                progressDialog.setMessage(getString(R.string.settings_backing_up));
                UsersSettingsBusiness.put(STSBusiness.getConfig(MainActivity.this), data, user.getId())
                        .then(new LuPromise.LuConsumer<LuVoidDbo>() {
                            @Override
                            public void execute(LuVoidDbo data) {
                                progressDialog.dismiss();
                                Toast.makeText(MainActivity.this, R.string.settings_backed_up, Toast.LENGTH_LONG).show();
                            }
                        }, new LuPromise.LuConsumer<LuPromise.LuPromiseError>() {
                            @Override
                            public void execute(LuPromise.LuPromiseError data) {
                                progressDialog.dismiss();
                                Toast.makeText(MainActivity.this, data.toString(), Toast.LENGTH_LONG).show();
                            }
                        });
            }
        });
    }
}
